package info.robit.dkpfetch;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;

public class MagicFetch extends JFrame {
    private JPanel contentPane = new JPanel();
    private JButton downloadButton;
    private JButton changeButton;
    private JLabel destinationPath;
    private JTextField websiteURL;
    private JLabel statusLabel;
    private JCheckBox launchAfterDownloadCheckBox;
    private JButton changeButton1;
    private JLabel wowLauncher;
    private JFileChooser chooser;
    private JFrame chooserFrame;

    Configuration config = new Configuration();

    public MagicFetch() {
        $$$setupUI$$$();
        destinationPath.setText(config.getProperty(Configuration.DIR));
        websiteURL.setText(config.getProperty(Configuration.URL));
        wowLauncher.setText(config.getProperty(Configuration.WOWBIN));
        if (config.getProperty(Configuration.AUTOSTART) != null) {
            launchAfterDownloadCheckBox.setSelected(true);
        }
        setResizable(true);
        setContentPane(contentPane);
        getRootPane().setDefaultButton(downloadButton);
        setTitle("MagicDKP Download v1.0  2009, NeoTron (neotron@gmail.com)");
        setSize(new Dimension(670, 180));

        downloadButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onOK();
            }
        });

        // call onExit() when cross is clicked
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                onExit();
            }
        });

        // Cancel on escape
        contentPane.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onExit();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

        changeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onChangeDestination();
            }
        });
        launchAfterDownloadCheckBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (launchAfterDownloadCheckBox.isSelected()) {
                    config.setProperty(Configuration.AUTOSTART, "true");
                } else {
                    config.unsetProperty(Configuration.AUTOSTART);
                }
            }
        });
        changeButton1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onSetWowBinary();
            }
        });
    }

    private void SetStatus(String status) {
        statusLabel.setText(status);
    }

    private void SetStatus(String status, Exception e) {
        statusLabel.setText(status + e.toString());
    }

    private void onOK() {
        synchronized (this) {
            Thread thread = new Thread(new FetchURL());
            thread.start();
        }
    }

    private void onExit() {
        try {
            config.save();
        } catch (IOException e) {
            // ignore
        }
        dispose();
        System.exit(0);
    }

    private void onChangeDestination() {
        chooser.setDialogType(JFileChooser.CUSTOM_DIALOG);
        chooser.setDialogTitle("Select the MagicDKP_Database Directory");
        chooser.setCurrentDirectory(new File(destinationPath.getText()));
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.showDialog(chooserFrame, "Select");
        if (chooser.getSelectedFile() != null) {
            String dir = chooser.getSelectedFile().toString();
            destinationPath.setText(dir);
            config.setProperty(Configuration.DIR, dir);
            try {
                config.save();
            } catch (IOException e) {
                SetStatus("Failed to save preferences...");
            }
        }
    }

    private void onSetWowBinary() {
        chooser.setDialogTitle("Select the World of Warcraft Launcher");
        chooser.setCurrentDirectory(new File(wowLauncher.getText()).getParentFile());
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.showDialog(chooserFrame, "Select");
        if (chooser.getSelectedFile() != null) {
            String file = chooser.getSelectedFile().toString();
            wowLauncher.setText(file);
            config.setProperty(Configuration.WOWBIN, file);
            try {
                config.save();
            } catch (IOException e) {
                SetStatus("Failed to save preferences...");
            }
        }
    }

    public static void main(String[] args) {
        MagicFetch frame = new MagicFetch();
        //    frame.pack();
        frame.setVisible(true);
    }

    private void createUIComponents() {
        chooser = new JFileChooser();
        chooserFrame = new JFrame();
        chooser.setDialogType(JFileChooser.CUSTOM_DIALOG);
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection ALL
     */
    private void $$$setupUI$$$() {
        createUIComponents();
        contentPane.setLayout(new com.intellij.uiDesigner.core.GridLayoutManager(1, 1, new Insets(10, 10, 10, 10), -1, -1));
        contentPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0), null, TitledBorder.LEFT, TitledBorder.DEFAULT_POSITION));
        final JPanel panel1 = new JPanel();
        panel1.setLayout(new com.intellij.uiDesigner.core.GridLayoutManager(4, 4, new Insets(0, 0, 0, 0), -1, -1));
        contentPane.add(panel1, new com.intellij.uiDesigner.core.GridConstraints(0, 0, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_BOTH, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        final JLabel label1 = new JLabel();
        label1.setText("Website URL:");
        panel1.add(label1, new com.intellij.uiDesigner.core.GridConstraints(0, 0, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JLabel label2 = new JLabel();
        label2.setText("Destination Path:");
        panel1.add(label2, new com.intellij.uiDesigner.core.GridConstraints(1, 0, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        destinationPath = new JLabel();
        destinationPath.setBackground(new Color(-3407668));
        destinationPath.setHorizontalTextPosition(11);
        destinationPath.setText("<please set destination path>");
        destinationPath.setToolTipText("The path to MagicDKP Database");
        panel1.add(destinationPath, new com.intellij.uiDesigner.core.GridConstraints(1, 1, 1, 2, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, new Dimension(350, -1), null, new Dimension(400, -1), 0, false));
        changeButton = new JButton();
        changeButton.setText("Change");
        changeButton.setToolTipText("Set the download destination directory. Typically you want to download to the MagicDKP_Database directory.");
        panel1.add(changeButton, new com.intellij.uiDesigner.core.GridConstraints(1, 3, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_EAST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, 1, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        websiteURL = new JTextField();
        websiteURL.setToolTipText("The URL to your EQDKP Plus getdkp.php file.");
        panel1.add(websiteURL, new com.intellij.uiDesigner.core.GridConstraints(0, 1, 1, 3, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_HORIZONTAL, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(150, -1), null, 0, false));
        statusLabel = new JLabel();
        statusLabel.setText("");
        panel1.add(statusLabel, new com.intellij.uiDesigner.core.GridConstraints(3, 0, 1, 2, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        downloadButton = new JButton();
        downloadButton.setText("Download");
        panel1.add(downloadButton, new com.intellij.uiDesigner.core.GridConstraints(3, 3, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_EAST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, 1, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JLabel label3 = new JLabel();
        label3.setText("WoW Path:");
        panel1.add(label3, new com.intellij.uiDesigner.core.GridConstraints(2, 0, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        wowLauncher = new JLabel();
        wowLauncher.setText("<...>");
        panel1.add(wowLauncher, new com.intellij.uiDesigner.core.GridConstraints(2, 1, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, new Dimension(250, -1), null, new Dimension(250, -1), 0, false));
        launchAfterDownloadCheckBox = new JCheckBox();
        launchAfterDownloadCheckBox.setText("Launch after download");
        panel1.add(launchAfterDownloadCheckBox, new com.intellij.uiDesigner.core.GridConstraints(2, 2, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, 1, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        changeButton1 = new JButton();
        changeButton1.setText("Change");
        panel1.add(changeButton1, new com.intellij.uiDesigner.core.GridConstraints(2, 3, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_EAST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, 1, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    }

    /**
     * @noinspection ALL
     */
    public JComponent $$$getRootComponent$$$() {
        return contentPane;
    }

    private class FetchURL implements Runnable {
        public void run() {
            URL url;
            try {
                url = new URL(websiteURL.getText());
            } catch (MalformedURLException e) {
                SetStatus("Invalid URL: ", e);
                return;
            }

            config.setProperty(Configuration.URL, websiteURL.getText());
            config.setProperty(Configuration.DIR, destinationPath.getText());
            try {
                config.save();
            } catch (IOException e) {
                SetStatus("Failed to save config file: ", e);
                return;
            }
            File output = new File(destinationPath.getText(), "dkp_list.lua");
            if (!output.exists()) {
                try {
                    if (!output.createNewFile()) {
                        SetStatus("Failed to create output file. Check file path and permissions.");
                    }
                } catch (IOException e) {
                    SetStatus("Failed to create file: ", e);
                    return;
                }
            }
            if (!output.canWrite()) {
                SetStatus("Can't open output file for writing. Check file permissions.");
                return;
            }
            SetStatus("Downloading DKP data...");
            try {
                BufferedReader input = new BufferedReader(new InputStreamReader(url.openStream()));
                StringBuilder data = new StringBuilder();
                long readData = 0;
                String line;
                while ((line = input.readLine()) != null) {
                    readData += line.length();
                    SetStatus("Downloading: " + readData + " bytes...");
                    data.append(line).append("\n");
                }

                String result = data.toString();
                FileWriter writer = new FileWriter(output);
                writer.write(result);
                writer.close();
                SetStatus("Download complete. Read " + result.length() + " bytes.");
            } catch (IOException e) {
                SetStatus("Fetch Failed: ", e);
                return;
            }
            if (launchAfterDownloadCheckBox.isSelected()) {
                try {
                    Process process = Runtime.getRuntime().exec(wowLauncher.getText());
                    System.exit(0);
                } catch (IOException e) {
                    SetStatus("Failed to start WoW: ", e);
                }
            }
        }
    }
}
